@@ -1,10 +1,10 @@
 ;;; eieio-speedbar.el -- Classes for managing speedbar displays.
 
 ;;;
-;; Copyright (C) 1999, 2000, 2001 Eric M. Ludlam
+;; Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007 Eric M. Ludlam
 ;;
 ;; Author: <zappo@gnu.org>
-;; RCS: $Id: eieio-speedbar.el,v 1.2 2001/02/20 03:24:58 youngs Exp $
+;; RCS: $Id: eieio-speedbar.el,v 1.4 2007/11/26 15:01:05 michaels Exp $
 ;; Keywords: oop, tools
 ;;
 ;; This program is free software; you can redistribute it and/or modify
@@ -18,12 +18,9 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, you can either send email to this
-;; program's author (see below) or write to:
-;;
-;;              The Free Software Foundation, Inc.
-;;              675 Mass Ave.
-;;              Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 ;;
 ;; Please send bug reports, etc. to zappo@gnu.org
 
@@ -42,6 +39,7 @@
 ;;
 ;; 1) Objects that will have lists of items that can be expanded
 ;;    should also inherit from the classes:
+;;  * `eieio-speedbar'                  - specify your own button behavior
 ;;  * `eieio-speedbar-directory-button' - objects that behave like directories
 ;;  * `eieio-speedbar-file-button'      - objects that behave like files
 ;;
@@ -100,22 +98,22 @@
 
 ;;; Support a way of adding generic object based modes into speedbar.
 ;;
-(defvar eieio-speedbar-key-map nil
-  "A Generic object based speedbar display keymap.")
-
 (defun eieio-speedbar-make-map ()
   "Make the generic object based speedbar keymap."
-  (setq eieio-speedbar-key-map (speedbar-make-specialized-keymap))
+  (let ((map (speedbar-make-specialized-keymap)))
 
-  ;; General viewing things
-  (define-key eieio-speedbar-key-map "\C-m" 'speedbar-edit-line)
-  (define-key eieio-speedbar-key-map "+" 'speedbar-expand-line)
-  (define-key eieio-speedbar-key-map "=" 'speedbar-expand-line)
-  (define-key eieio-speedbar-key-map "-" 'speedbar-contract-line)
+    ;; General viewing things
+    (define-key map "\C-m" 'speedbar-edit-line)
+    (define-key map "+" 'speedbar-expand-line)
+    (define-key map "=" 'speedbar-expand-line)
+    (define-key map "-" 'speedbar-contract-line)
 
-  ;; Some object based things
-  (define-key eieio-speedbar-key-map "C" 'eieio-speedbar-customize-line)
-  )
+    ;; Some object based things
+    (define-key map "C" 'eieio-speedbar-customize-line)
+    map))
+
+(defvar eieio-speedbar-key-map (eieio-speedbar-make-map)
+  "A Generic object based speedbar display keymap.")
 
 (defvar eieio-speedbar-menu
   '([ "Edit Object/Field" speedbar-edit-line t]
@@ -131,6 +129,7 @@
     )
   "Menu part in easymenu format used in speedbar while browsing objects.")
 
+;; Note to self:  Fix this silly thing!
 (defalias 'eieio-speedbar-customize-line  'speedbar-edit-line)
 
 (defun eieio-speedbar-create (map-fn map-var menu-var modename fetcher)
@@ -167,7 +166,7 @@ MAP-FN, MAP-VAR, MENU-VAR, MODENAME, and FETCHER are the same as
   (speedbar-add-mode-functions-list
    (list modename
 	 '(speedbar-item-info . eieio-speedbar-item-info)
-	 '(speedbar-line-path . eieio-speedbar-line-path))))
+	 '(speedbar-line-directory . eieio-speedbar-line-path))))
 
 (defun eieio-speedbar-buttons (dir-or-object depth fetcher)
   "Create buttons for the speedbar display.
@@ -251,30 +250,33 @@ and take the apropriate action."
 	       "The type of expansion button used for objects of this class.
 Possible values are those symbols supported by the `exp-button-type' argument
 to `speedbar-make-tag-line'."
-	       :allocation class)
+	       :allocation :class)
    (buttonface :initform speedbar-tag-face
 	       :type (or symbol face)
 	       :documentation
 	       "The face used on the textual part of the button for this class.
 See `speedbar-make-tag-line' for details."
-	       :allocation class)
+	       :allocation :class)
    (expanded :initform nil
 	     :type boolean
 	     :documentation
 	     "State of an object being expanded in speedbar.")
    )
   "Class which provides basic speedbar support for child classes.
-Add one of thie child classes to this class to the parent list of a class.")
+Add one of thie child classes to this class to the parent list of a class."
+  :abstract t)
 
 (defclass eieio-speedbar-directory-button (eieio-speedbar)
   ((buttontype :initform angle)
    (buttonface :initform speedbar-directory-face))
-  "Class providing support for objects which behave like a directory.")
+  "Class providing support for objects which behave like a directory."
+  :abstract t)
 
 (defclass eieio-speedbar-file-button (eieio-speedbar)
   ((buttontype :initform bracket)
    (buttonface :initform speedbar-file-face))
-  "Class providing support for objects which behave like a directory.")
+  "Class providing support for objects which behave like a directory."
+  :abstract t)
 
 
 ;;; Methods to eieio-speedbar-* which do not need to be overriden
@@ -288,13 +290,21 @@ Argument DEPTH is the depth at which the tag line is inserted."
   (let ((children (eieio-speedbar-object-children object))
 	(exp (oref object expanded)))
     (if (not children)
-	(speedbar-make-tag-line (oref object buttontype)
-				?? nil nil
-				(eieio-speedbar-object-buttonname object)
-				'eieio-speedbar-object-click
-				object
-				(oref object buttonface)
-				depth)
+	(if (eq (oref object buttontype) 'expandtag)
+	    (speedbar-make-tag-line 'statictag
+				    ?  nil nil
+				    (eieio-speedbar-object-buttonname object)
+				    'eieio-speedbar-object-click
+				    object
+				    (oref object buttonface)
+				    depth)
+	  (speedbar-make-tag-line (oref object buttontype)
+				  ?  nil nil
+				  (eieio-speedbar-object-buttonname object)
+				  'eieio-speedbar-object-click
+				  object
+				  (oref object buttonface)
+				  depth))
       (speedbar-make-tag-line (oref object buttontype)
 			      (if exp ?- ?+)
 			      'eieio-speedbar-object-expand
@@ -309,7 +319,7 @@ Argument DEPTH is the depth at which the tag line is inserted."
 
 (defmethod eieio-speedbar-child-make-tag-lines ((object eieio-speedbar) depth)
   "Base method for creating tag lines for non-object children."
-  (error "You must implement `eieio-speedbar-child-make-tag-lnes' for %s"
+  (error "You must implement `eieio-speedbar-child-make-tag-lines' for %s"
 	 (object-name object)))
 
 (defmethod eieio-speedbar-expand ((object eieio-speedbar) depth)
@@ -321,7 +331,7 @@ OBJECT."
 	   (mapcar (lambda (car)
 		     (eieio-speedbar-make-tag-line car depth))
 		   children))
-	  (t (eieio-speedbar-child-make-tag-lines object depth)))))
+	  (children (eieio-speedbar-child-make-tag-lines object depth)))))
 
 
 ;;; Speedbar specific function callbacks.
@@ -372,7 +382,7 @@ Optional argument DEPTH is the current depth of the search."
 	(progn
 	  (beginning-of-line)
 	  (when (looking-at "^\\([0-9]+\\):")
-	    (setq depth (string-to-int (match-string 1))))))
+	    (setq depth (string-to-number (match-string 1))))))
     (when depth
       (while (and (not (object-p (speedbar-line-token)))
 		  (> depth 0))
@@ -388,7 +398,7 @@ Optional DEPTH is the depth we start at."
 	(progn
 	  (beginning-of-line)
 	  (looking-at "^\\([0-9]+\\):")
-	  (setq depth (string-to-int (match-string 1)))))
+	  (setq depth (string-to-number (match-string 1)))))
     ;; This whole function is presently bogus.  Make it better later.
     (let ((tok (eieio-speedbar-find-nearest-object depth)))
       (if (object-p tok)
